package com.bimbr.clisson.util;

import java.util.Map;
import java.util.Set;

/**
 * Utility functions to verify arguments.
 *
 * @author mmakowski
 * @since 1.0.0 
 */
public final class Arguments {
    /**
     * Checks that argument is not null.
     * @param arg the argument whose value is to be checked
     * @param argName the name of the argument, for diagnostics
     * @return the value of {@code arg}
     * @throws IllegalArgumentException if {@code arg} is null
     * @since 1.0.0
     */
    public static <T> T nonNull(T arg, String argName) {
        if (arg == null) throw new IllegalArgumentException(argName + " is null");
        return arg;
    }

    /**
     * Checks that argument is a non-empty string.
     * @param arg the argument whose value is to be checked
     * @param argName the name of the argument, for diagnostics
     * @return the value of {@code arg}
     * @throws IllegalArgumentException if {@code arg} is an empty string
     * @since 1.0.0
     */
    public static String nonEmpty(String arg, String argName) {
        nonNull(arg, argName);
        if (arg.length() == 0) throw new IllegalArgumentException(argName + " is an empty string");
        return arg;
    }

    /**
     * Checks that argument is a positive integer. 
     * @param arg the argument whose value is to be checked
     * @param argName the name of the argument, for diagnostics
     * @return the value of {@code arg}
     * @since 1.0.0
     */
    public static int positive(int arg, String argName) {
        if (arg <= 0) throw new IllegalArgumentException(argName + " must be positive but was " + arg);
        return arg;
    }
    
    /**
     * Checks that argument is a non-empty set.
     * @param arg the argument whose value is to be checked
     * @param argName the name of the argument, for diagnostics
     * @return the value of {@code arg}
     * @throws IllegalArgumentException if {@code arg} is null or an empty set
     * @since 1.0.0
     */
    public static <E> Set<E> nonEmpty(Set<E> arg, String argName) {
        nonNull(arg, argName);
        if (arg.isEmpty()) throw new IllegalArgumentException(argName + " is an empty set");
        return arg;
    }
    
    /**
     * Checks that map does not contain null keys and null values.
     * @param arg the map whose keys and values are to be verified
     * @param argName the name of the argument, for diagnostics
     * @return hte value of {@code arg}
     * @throws IllegalArgumentException if {@code arg} is null or contains null keys or values
     * @since 1.0.0
     */
    public static <K, V> Map<K, V> mapWithNonNullKeysAndValues(Map<K, V> arg, String argName) {
        nonNull(arg, argName);
        try {
            if (arg.containsKey(null)) throw new IllegalArgumentException(argName + " contains null key: " + arg.toString());
        } catch (NullPointerException e) {
            // some implementations of Map (e.g. TreeMap) throw NPE when null is passed to containsKey() -- assume in this case the map doesn't contain null
            // OK
        }
        if (arg.containsValue(null)) throw new IllegalArgumentException(argName + " contains null value: " + arg.toString());
        return arg;
    }
    
    /**
     * Checks that set does not contain null values.
     * @param arg the set whose elements are to be verified
     * @param argName the name of the argument, for diagnostics
     * @return hte value of {@code arg}
     * @throws IllegalArgumentException if {@code arg} is null or contains null values
     * @since 1.0.0
     */
    public static <E> Set<E> setWithNonNullElements(Set<E> arg, String argName) {
        nonNull(arg, argName);
        try {
            if (arg.contains(null)) throw new IllegalArgumentException(argName + " contains null: " + arg.toString());
        } catch (NullPointerException e) {
            // some implementations of Set (e.g. TreeSet) throw NPE when null is passed to contains() -- assume in this case the set doesn't contain null
            // OK
        }
        return arg;
    }
    
    private Arguments() {}
}